/* ============================================================
*
* This file is a part of the rekonq project
*
* Copyright (C) 2009 Nokia Corporation <qt-info@nokia.com>
* Copyright (C) 2009 by Andrea Diamantini <adjam7 at gmail dot com>
*
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License or (at your option) version 3 or any later version
* accepted by the membership of KDE e.V. (or its successor approved
* by the membership of KDE e.V.), which shall act as a proxy 
* defined in Section 14 of version 3 of the license.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
* ============================================================ */


// Self Includes
#include "websnap.h"
#include "websnap.moc"

// KDE Includes
#include <KDebug>
#include <KStandardDirs>

// Qt Includes
#include <QSize>
#include <QWebFrame>
#include <QWebSettings>
#include <QPainter>
#include <QTimer>
#include <QFile>


#define WIDTH  200
#define HEIGHT 150


WebSnap::WebSnap(const QUrl &url)
    : QObject()
{
    m_url = url;

    // this to not register websnap history
    m_page.settings()->setAttribute(QWebSettings::PrivateBrowsingEnabled, true);
    
    // this to not let this page open other windows
    m_page.settings()->setAttribute(QWebSettings::PluginsEnabled, false);
    m_page.settings()->setAttribute(QWebSettings::JavascriptEnabled, false);

    connect(&m_page, SIGNAL(loadFinished(bool)), this, SLOT(saveResult(bool)));
    QTimer::singleShot(0, this, SLOT(load()));
}


WebSnap::~WebSnap()
{
}


void WebSnap::load()
{
    m_page.mainFrame()->load(m_url);
}


QPixmap WebSnap::renderPreview(const QWebPage &page,int w, int h)
{
    // prepare page
    page.mainFrame()->setScrollBarPolicy(Qt::Vertical, Qt::ScrollBarAlwaysOff); // Why it doesn't work with one setScrollBarPolicy?
    page.mainFrame()->setScrollBarPolicy(Qt::Vertical, Qt::ScrollBarAlwaysOff); // bug in qtwebkit ?
    page.mainFrame()->setScrollBarPolicy(Qt::Horizontal, Qt::ScrollBarAlwaysOff);
    page.mainFrame()->setScrollBarPolicy(Qt::Horizontal, Qt::ScrollBarAlwaysOff);

    // find the best size
    QSize size;
    if (page.viewportSize().width() && page.viewportSize().height())
    {
        size = page.viewportSize();
    }
    else
    {
        int width = page.mainFrame()->contentsSize().width();
        if (width < 640) width = 640;
        size = QSize(width,width*((0.0+h)/w));
        page.setViewportSize(size);
    }
    
    // create the page image
    QImage pageImage = QImage(size, QImage::Format_ARGB32_Premultiplied);
    pageImage.fill(Qt::transparent); 
    // render it
    QPainter p(&pageImage);
    page.mainFrame()->render(&p);
    p.end();
    pageImage = pageImage.scaled(w, h, Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation);

    // restore page settings
    page.mainFrame()->setScrollBarPolicy(Qt::Horizontal, Qt::ScrollBarAsNeeded);
    page.mainFrame()->setScrollBarPolicy(Qt::Horizontal, Qt::ScrollBarAsNeeded);
    page.mainFrame()->setScrollBarPolicy(Qt::Vertical, Qt::ScrollBarAsNeeded);
    page.mainFrame()->setScrollBarPolicy(Qt::Vertical, Qt::ScrollBarAsNeeded);

    return QPixmap::fromImage(pageImage);
}


void WebSnap::saveResult(bool ok)
{
    // crude error-checking
    if (!ok) 
    {
        kDebug() << "Error loading site..";
        return;
    }

    m_image = renderPreview(m_page, WIDTH, HEIGHT);
    emit finished();
}



QString WebSnap::snapTitle()
{
    return m_page.mainFrame()->title();
}


QUrl WebSnap::snapUrl()
{
    return m_url;
}


QPixmap WebSnap::previewImage()
{
    return m_image;
}
